module Filter where

import SimpleParser
import Char
import List
import Puff
import Text.Regex.Posix
import PackedString

------------------------
-- Filter implementation
------------------------

data Filter = FilterSub String | FilterAuthor String | FilterCategory Category  | 
    FilterKeyword String | FilterRealName String | FilterNot Filter |
    FilterOr [Filter] | FilterAnd [Filter] | FilterISub String | FilterPrivate | FilterTrue |
    FilterRegex Regex

parseFilter :: Monad m => String -> m Filter
parseFilter = parser (between spaces eof rmp) where
    rmp = do
	fa <- mp 
	r <- rmp'
	case r of
	    [] -> return fa
	    _ -> return $ FilterOr (fa:r)
    rmp' = (do
	char ';'
	spaces
	v <- mp
	r <- rmp'
	return (v:r)) <|> return []
    mp = many1 fp >>= \v -> case v of
	    [x] -> return x
	    xs -> return $ FilterAnd xs
    fp = nf <|> flt <|> pr <|> bool <|> fmap FilterSub (token string) 
    pr = do
	char '('
	spaces
	v <- rmp
	char ')'
	spaces
	return v
    nf = char '!' >> fmap FilterNot fp
    string = qstring <|> many1 (noneOf " \t\n~'!();%")
    qstring = between (char '\'') (char '\'') $ many ((char '\'' >> char '\'' >> return '\'') <|> noneOf "'")
    bool = do
	char '%'
	c <- sat isAlpha
	spaces
	return $ b c
    flt = do
	char '~'
	c <- sat isAlpha
	option ':' (char ':')
	s <- string
	spaces
	return $ z c s
    z 'a' s = FilterAuthor s
    z 's' s = FilterRealName s
    z 'c' s = FilterCategory (catParseNew s)
    z 't' s = FilterCategory (catParseNew s) -- TODO
    z 'k' s = FilterKeyword s
    z 'i' s = FilterISub s
    z _ s = FilterSub s
    b 'p' = FilterPrivate
    b 'P' = FilterNot FilterPrivate
    b _ = FilterTrue


applyFilterStack :: [Filter] -> [(Int, Puff)] -> [(Int, Puff)]
applyFilterStack fs = concatMap f where
    f a@(_,p) | all (flip filterAp p) fs = [a]
    f _ = []


siglookup c (Unverifyable (Key n _):_) | c == n = True
siglookup c (Signed (Key n _):_) | c == n = True
siglookup c (_:xs) = siglookup c xs
siglookup _ [] = False



filterAp :: Filter -> Puff -> Bool
filterAp (FilterNot f) p = not $ filterAp f p
filterAp (FilterRealName s) p = case getFragmentString p f_messageSender of
    (Just s') | s' == packString s -> True
    _ -> False
filterAp (FilterAuthor c) (Puff {signature = sig})  = siglookup c sig 
filterAp (FilterKeyword k) p = any (k `isPrefixOf`) $ map unpackPS (getFragmentStrings p f_messageKeyword)
filterAp (FilterCategory c) (Puff {cats = cs}) = any (`subCategory` c) cs
filterAp (FilterSub s) p = case fmap unpackPS (getFragmentString p f_messageBody) of
    Just b | any (s `isPrefixOf`) (tails b) ->  True
    _ -> False
filterAp (FilterISub s) p = case fmap unpackPS (getFragmentString p f_messageBody) of
    Just b | any ((map toLower s) `isPrefixOf`) (tails (map toLower b)) ->  True
    _ -> False
filterAp (FilterAnd fs) p = all (`filterAp` p) fs
filterAp (FilterOr fs) p = any (`filterAp` p) fs
filterAp FilterTrue _ = True
filterAp FilterPrivate _ = False

showFilter :: Filter -> String
showFilter (FilterSub s) = s
showFilter (FilterISub s) = "~i:" ++ s
showFilter (FilterAuthor a) = "~a:" ++  a
showFilter (FilterRealName s) = "~s:" ++ s
showFilter (FilterCategory c) = "~c:" ++ catShowNew c
showFilter (FilterKeyword c) = "~k:" ++ c
showFilter (FilterNot f) = "!" ++ showFilter f
showFilter (FilterAnd fs) = concat (intersperse " " (map showFilter fs))
showFilter (FilterOr fs) = "(" ++ (concat $ intersperse " ; " (map showFilter fs)) ++ ")"
showFilter FilterPrivate = "%p"
showFilter FilterTrue = "%t"
--showFilter _ = ""
