module Help(bugMsg, helpTable, usageHeader, usageTrailer) where

import GenUtil(buildTableRL, indentLines)

{-# NOINLINE usageHeader #-}
{-# NOINLINE usageTrailer #-}
{-# NOINLINE bugMsg #-}
{-# NOINLINE helpTable #-}

usageHeader :: String
usageTrailer :: String
bugMsg :: String
helpTable :: String

usageHeader = "Usage: ginsu [OPTION...] categories..."

usageTrailer = unlines [
    "\nTo get started, generate a configuration file using the following command:",
    "   mkdir ~/.gale ; ginsu -s > ~/.gale/ginsu.config",
    "Edit the configuration file and run ginsu!",
    "",
    "The homepage is at:",
    "   <http://repetae.net/john/computer/ginsu/>",
    "To report a bug go to:",
    "   <http://bugs.ofb.net/cgi-bin/bugzilla/enter_bug.cgi?product=Ginsu>",
    "When reporting a bug, include the last few lines of ~/.gale/ginsu.errorlog if it seems appropriate."
    ]

bugMsg = unlines [
    "There has been an internal error",
    "verify the output of ginsu --checkconfig and",
    "please inform <john@ugcs.caltech.edu> or file a bug report at",
    "<http://bugs.ofb.net/cgi-bin/bugzilla/enter_bug.cgi?product=Ginsu>",
    "Include the following text in any error report:"
    ]


keys = buildTableRL [
    ("Keys:", ""),
    ("<F1>", "Help Screen"),
    ("<F2>", "Main Screen"),
    ("<F3>", "Presence Status Screen"),
    ("",""),
    ("j","next puff"),
    ("k","previous puff"),
    ("<Home>","first puff"),
    ("<End>","last puff"),
    ("<Enter> <C-E>","forward one line"),
    ("<BackSpace> <C-Y>","back one line"),
    ("<C-F> <PgDown>","forward one page"),
    ("<C-B> <PgUp>","back one page"),
    ("<C-D>","forward one half-page"),
    ("<C-U>","back one half-page"),
    ("",""),
    ("d","Show puff details"),
    ("",""),
    ("c ~","add new filter"),
    ("u","pop one filter"),
    ("U","pop all filters"),
    ("!","invert filter at top of stack"),
    ("x","swap top two filters on stack"),
    ("a","filter to current author"),
    ("t","filter to current thread"),
    ("T","filter to strict thread"),
    ("m[1-9]","set mark - save current filter stack at given key"),
    ("[1-9]","recall filter stack saved here with 'm'"),
    ("C[1-9]","recall filter stack and add it to current filter stack"),
    ("",""),
    ("f","follow-up to category"),
    ("p","compose new puff"),
    ("r","reply to author"),
    ("g","group reply, sends to recipients and author of puff"),
    ("R","resend selected puff"),
    ("N","modify public presence string"),
    ("",""),
    ("<C-R>","reconnect to all servers"),
    ("E","edit and reload configuration file"),
    ("q Q","quit program"),
    ("<C-L>","redraw screen")
    ]

helpTable = unlines keys ++ filters

filters = "\nFilters:\n" ++ indentLines 4 (paragraph 70 filterDoc) ++ "\n" ++ unlines (buildTableRL [
    ("Primitive Filters:",""),
    ("~a","author"),
    ("~c","category"),
    ("~i","case insensitive substring search"),
    ("~k","keyword"),
    ("~s","puffers real name"),
--    ("~t","thread - transitive closure of categories"),
    ("(default)","substring search")
    ]) ++ "\n" ++ filterExamples


filterDoc = "Filters are implemented as boolean expressions. The building blocks are primitives which usually begin with ~ and are combined with semicolon for disjunction, space for conjunction, exclamation point for negation and parenthesis for grouping. The current filter stack is displayed on the bottom line of the screen when it is nonempty, the filters on the stack are conjoined together to determine which puffs are visible"


paragraph maxn xs = drop 1 (f maxn (words xs)) where
    f n (x:xs) | lx <- length x + 1, lx < n = (' ':x) ++ f (n - lx) xs 
    f n (x:xs) = '\n': (x ++ f (maxn - length x) xs)
    f n [] = "\n"
    

filterExamples = "Filter Examples:\n"  ++ indentLines 4 (unlines es) where
    es = [
	"(~a:john@ugcs.caltech.edu) - all puffs by john@ugcs",
	"(~c:pub.tv.buffy ~a:jtr@ofb.net) - puffs from jtr and to pub.tv.buffy",
	"(ginsu ; ~c:pub.gale.ginsu) - puffs containing the word ginsu or directed to pub.gale.ginsu",
	"(!~k:spoil) - no spoilers",
	"(~c:pub.tv.buffy !~c:pub.meow) - puffs to buffy which are not about cats"
	]
